(***************************************************************)
(***************************************************************)
(** FeynRules model file for top EFT couplings induced  ********)
(** by a scalar top partner and a fermion singlet (DM).  *******)
(** Only the operators relevant for q q > t t are included. ****)
(** The on-shell (physical) basis is assumed with the         **)
(** coefficients in the heavy mass limit (mST,mChi >> mTop,s) **)
(** computed using Matchete 0.1.5                           ****)
(******                                                        *)
(****** A. Lessa                                               *)
(******                                                        *)
(****** Feel free to use this model file!                      *)
(***************************************************************)
(***************************************************************)

M$ModelName = "Top-EFT";

M$Information = {Authors -> {"Andre Lessa"}, 
                 Version -> "1.0",
                 Date -> "28.08.2023"};

FeynmanGauge = True;

M$InteractionOrderHierarchy = {  {NP, 1}, {QCD, 2}, {QED,3} };

(* ************************** *)
(* *****   Parameters   ***** *)
(* ************************** *)

M$Parameters = {

(* New Physics (UV) Parameters *)
  yDM  == { TeX->Subscript[y,DM],  ComplexParameter -> False, ParameterType->External, Value->1.0, InteractionOrder->{NP ,1}, Description->"Dark fermion - heavy stop - SM RH top Yukawa"},


(* EFT coefficients *)
  A0  == { TeX->Subscript[A,0],  
          ComplexParameter -> False, 
          ParameterType->Internal, 
          Value-> (-gs*yDM^2*MT/(384.*Pi^2))*If[1-mChi^2/mST^2 < 0.1, mChi^2/(5*mST^4)-4*mChi/(5*mST^3) + 11/(10*mST^2),(mST^6/mChi^6 + 2*(1 - (3*mST^4)/mChi^4) + (3*mST^2*(1 + 4*Log[mST/mChi]))/mChi^2)/(mChi^2*(1 - mST^2/mChi^2)^4)],           
          InteractionOrder->{{NP,2},{QCD,1}},
          Description->"Effective on-shell g-top-top coupling (=MT*Cg)"
        },

  B0 == {TeX->Subscript[B,0],
           ComplexParameter -> False, 
           ParameterType->Internal, 
           Value -> (gs^2*yDM^2/(3456.*Pi^2))*If[1-mChi^2/mST^2 < 0.1,mChi^2/(10*mST^4)-4*mChi/(5*mST^3)+11/(5*mST^2),(6*mChi^6*Log[mChi^2/mST^2] - 11*mChi^6 + 18*mST^2*mChi^4 - 9*mST^4*mChi^2 + 2*mST^6)/((mChi^2-mST^2)^4)],
           InteractionOrder->{{NP,2},{QCD,2}},
           Description->"Auxiliar coupling for computing coefficients (=Cq/6)"
  }  
  
};


(* ************************** *)
(* **** Particle classes **** *)
(* ************************** *)
M$ClassesDescription = {


(* The dark fermion *)
F[16] == { 
         ClassName -> chi, 
         SelfConjugate-> True, 
         QuantumNumbers -> {}, 
         Mass->{mChi,  100.0},  
         Width->{wChi,0.}, 
         ParticleName-> "~chi", 
         PDG->5000012, 
         PropagatorLabel->"chi", 
         PropagatorType->Straight, 
         ParticleName     -> {"~chi" },
         FullName         -> {"Dark fermion"} },

(* Colored Scalar *)
S[5] == { 
          ClassName        -> ST,
          Indices          -> {Index[Colour]},
          SelfConjugate    -> False,
          Unphysical       -> False,
          Mass             -> {mST, 400.},
          Width            -> {wST, 1.},
          QuantumNumbers   -> {Q -> 2/3},
          PropagatorLabel  -> {"ST"},
          PropagatorType   -> ScalarDash, 
          PDG              -> {5000002},
          ParticleName     -> {"~ST" },
          AntiParticleName -> {"~ST~"},
          FullName         -> {"Colored Scalar"} }
};


(* ************************** *)
(* *****   Lagrangian   ***** *)
(* ************************** *)

(* Convenience function *)
Gammamunu[mu_,nu_,sp_,sp2_]:=Module[{sp1},(1/2)*(Ga[mu,sp,sp1]*Ga[nu,sp1,sp2]-Ga[nu,sp,sp1]*Ga[mu,sp1,sp2])];


(* gauge kinetic and mass terms *)

Lkin :=  Block[{mu,aa,bb},
               I*chibar.Ga[mu].DC[chi, mu] - (mChi/2) chibar.chi  + DC[STbar[aa],mu] DC[ST[aa],mu] -  mST^2 STbar[bb] ST[bb]
              ];

(* DM-Scalar-Top coupling *)
LYDM := Block[{cc,sp1,sp2},
            -yDM*(STbar[cc] ProjP[sp1,sp2] chibar[sp1].uq[sp2,3,cc])
            ];


(* EFT operator alphaOuG: G_{\mu\nu} tbar.sigma[\mu,\nu].Ta.t *)
(* Psi^2 X H *)
LA0 := Block[{lag,mu,nu,sp1,sp2,aa,cc1,cc2},
      lag= ExpandIndices[
                         I*A0*FS[G,mu,nu,aa]*(uqbar[sp1,3,cc1].uq[sp2,3,cc2])*T[aa,cc1,cc2]*Gammamunu[mu,nu,sp1,sp2]
                        ]
];

(* EFT operator QLQL-tRtR + dRdR-tRtR + uRuR-tRtR  *)
(* Four-quark *)
LB0 := Block[{aa,sp1,sp2,sp3,sp4,ff1,ff2,cc1,cc2,cc3,cc4,mu,lagh},
       lagh=ExpandIndices[
                          6*T[aa,cc2,cc1]*T[aa,cc3,cc4]*B0*(
                            uRbar[sp1,3,cc2].uR[sp2,3,cc1]*uqbar[sp3,ff1,cc3].uq[sp4,ff1,cc4]
                          + uRbar[sp1,3,cc2].uR[sp2,3,cc1]*dqbar[sp3,ff2,cc3].dq[sp4,ff2,cc4]
                            )*Ga[mu,sp1,sp2]*Ga[mu,sp3,sp4]
                          ]       
];

(* Total EFT lagrangian *)

LNP := Lkin + LYDM + HC[LYDM] + LA0 + LB0;
